
--- Sniper scope test
-- Charles Mallah, Jan 2018
-- Open license, do as you wish

function love.load()

	image          = love.graphics.newImage("screenshot3.png")

  width          = love.graphics:getWidth()
  height         = love.graphics:getHeight()
  
	canvas         = love.graphics.newCanvas(width,height)
  
  crosshairPc    = 0.3
  zoom           = 2
  scopeSize      = 50
  
	on             = true
  mousex, mousey = 0, 0
  hairSize       = scopeSize*crosshairPc
  love.mouse.setVisible( false )
end

function love.update(dt)	
  mousex, mousey = love.mouse.getPosition()  
end

local function myStencilFunction()
  love.graphics.circle("fill", mousex, mousey, scopeSize)
end

function love.draw()
  
  -- Here would be all your rendering pipe of the visible game area
	love.graphics.setCanvas(canvas)
	love.graphics.draw(image) -- For now, background image
  love.graphics.setCanvas()
  
  -- This can be drawn as is
  love.graphics.draw(canvas)
  
  -- Now the scope zoom
	if on then
    
    translatex = mousex
    translatey = mousey
  
    -- Each pixel touched by the circle will have its stencil value set to 1, rest is 0
    love.graphics.stencil(myStencilFunction, "replace", 1)

    -- Configure the stencil test to only allow rendering on pixels whose stencil value is equal to 0
    -- == every pixel except ones that were touched by the circle drawn as a stencil
    love.graphics.setStencilTest("equal", 1)
    
    -- Translate by the mouse position and scale by zoom level
    love.graphics.push("transform")
    love.graphics.translate(translatex, translatey)
    love.graphics.scale(zoom, zoom)
    love.graphics.translate(-translatex, -translatey)
    love.graphics.draw(canvas)
    love.graphics.pop()
    
    love.graphics.setStencilTest()
    
    -- Draw a crude 'scope' (use a graphic here!)
    love.graphics.setColor(0, 0, 0, 255)
    love.graphics.circle("line", mousex, mousey, scopeSize)
    love.graphics.print("x"..zoom, mousex+scopeSize-10, mousey+scopeSize-10)
    
	end
  
  -- Cross-hair
  love.graphics.line(mousex-hairSize, mousey, mousex+hairSize, mousey)
  love.graphics.line(mousex, mousey-hairSize, mousex, mousey+hairSize)
  
  love.graphics.setColor(255, 255, 255, 255)
end


function love.keypressed( key )
	if key == "z" then
		on = not on
	end
  if key == "1" then
    zoom = 1
  elseif key == "2" then
    zoom = 2
  elseif key == "3" then
    zoom = 5
  elseif key == "4" then
    zoom = 10
  end
  
  if key == "-" then
    scopeSize      = math.max(scopeSize - 20, 30)
    hairSize       = scopeSize*crosshairPc
  elseif key == "=" then
    scopeSize      = math.min(scopeSize + 20, 150)
    hairSize       = scopeSize*crosshairPc
  end
  
end
